import 'dotenv/config';

import { createLog } from '../log';
import type { Logger } from 'winston';
import { Database } from 'sqlite3';
import { envString } from '../env';

interface Photo {
	name : string;
	original? : string;
	hash : string;
	width : number;
	height : number;
	format? : string;
	filmstock? : string;
	location? : string;
	discovered ? : number;
	created? : number;
	updated? : number;
	bsky? : boolean;
	deleted? : boolean;
	score? : number;
}

export class DB {
	private db : Database;
	private log : Logger;

	constructor () {
		this.log = createLog('db');
		this.db = new Database(envString('DB', 'data/site.db'));
	}

	private async run (query : string, args : any[] = null) {
		return new Promise((resolve : Function, reject : Function) => {
			return this.db.run(query, args, (err : Error, rows : any[]) => {
				if (err) return reject(err);
				return resolve(true);
			});
		});
	}

//CASE WHEN LOWER(active) = 'true' THEN 1 ELSE 0 END AS active_bool
	public async create (photo : Photo) {
		const keys : string[] = Object.keys(photo);
		const query : string = `INSERT INTO photos (${keys.join(',')}) VALUES (${keys.map(el=>'?').join(',')});`;
		const values : any[] = [];
		for (let key of keys) {
			if (typeof (photo as any)[key] === 'boolean') {
				values.push((photo as any)[key] ? 1 : 0);
			} else {
				values.push((photo as any)[key]);
			}
		}
		try {
			await this.run(query, values);
			this.log.info(`Inserted new photo`)
		} catch (err) {
			this.log.error(`Error inserting record into photos`, err);
		}
	}

}

module.exports = { DB };
export type { Photo };