import Handlebars from 'handlebars';
import helpers from 'handlebars-helpers';
import { readFile, readdir } from 'fs/promises';
import { join, parse, basename } from 'path';
import { createLog } from '../log';
import type { Logger } from 'winston';

helpers();

export class Templates {
	private log : Logger;
	private dir : string;
	private templates : any = {};

	constructor (dir : string = './views') {
		this.dir = dir;
		this.log = createLog('tmpl');
	}

	public async build () {
		const partialsPath : string = join(this.dir, 'partials');
		let partials : string[] = [];
		let templates : string[] = [];
		let text : string;
		let name : string;

		try {
			partials = await readdir(partialsPath);
		} catch (err) {
			this.log.error(err);
		}

		partials = partials.map(el => join(partialsPath, el));

		for (let partial of partials) {
			name = parse(basename(partial)).name;
			try {
				text = await readFile(partial, 'utf8');
			} catch (err) {
				this.log.error(err);
				continue;
			}
			Handlebars.registerPartial(name, text);
			this.log.info(`[partial] ${name}`);
		}

		try {
			templates = await readdir(this.dir);
		} catch (err) {
			this.log.error(err);
		}

		templates = templates
			.filter(el => el.indexOf('.hbs') !== -1 || el.indexOf('.handlebars') !== -1)
			.map(el => join(this.dir, el));

		for (let template of templates) {
			name = parse(basename(template)).name;
			try {
				text = await readFile(template, 'utf8');
			} catch (err) {
				this.log.error(err);
				continue;
			}
			this.templates[name] = Handlebars.compile(text);
			this.log.info(`[template] ${name}`);
		}
	}

	public render (name : string, data : any) : string {
		const keys : string[] = Object.keys(data);
		this.log.info(`[render] ${name} with keys { ${keys.join(', ')} }`);
		return this.templates[name](data);
	}
}

module.exports = { Templates };

